@@ -60,7 +60,7 @@ OdbxBackend::OdbxBackend( const string& suffix )
 
         	if( getArg( "host" ).size() > 0 )
         	{
-        		L.log( m_myname + " WARNING: Using deprecated opendbx-host parameter", Logger::Warning );
+        		g_log.log( m_myname + " WARNING: Using deprecated opendbx-host parameter", Logger::Warning );
         		stringtok( m_hosts[READ], getArg( "host" ), ", " );
         		m_hosts[WRITE] = m_hosts[READ];
         	}
@@ -75,7 +75,7 @@ OdbxBackend::OdbxBackend( const string& suffix )
         }
         catch( std::exception& e )
         {
-        	L.log( m_myname + " OdbxBackend(): Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " OdbxBackend(): Caught STL exception - " + e.what(),  Logger::Error );
         	throw( PDNSException( "Fatal: STL exception" ) );
         }
 }
@@ -93,14 +93,14 @@ OdbxBackend::~OdbxBackend()
 
 
 
-bool OdbxBackend::getDomainInfo( const DNSName& domain, DomainInfo& di )
+bool OdbxBackend::getDomainInfo( const DNSName& domain, DomainInfo& di, bool getSerial )
 {
         const char* tmp;
 
 
         try
         {
-        	DLOG( L.log( m_myname + " getDomainInfo()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " getDomainInfo()", Logger::Debug ) );
 
         	string stmt = getArg( "sql-zoneinfo" );
         	string& stmtref = strbind( ":name", escape( domain.makeLowerCase().toStringRootDot(), READ ), stmt );
@@ -119,7 +119,7 @@ bool OdbxBackend::getDomainInfo( const DNSName& domain, DomainInfo& di )
         		di.backend = this;
         		di.serial = 0;
 
-        		if( ( tmp = odbx_field_value( m_result, 6 ) ) != NULL )
+        		if( getSerial && ( tmp = odbx_field_value( m_result, 6 ) ) != NULL )
         		{
         			SOAData sd;
 
@@ -141,7 +141,12 @@ bool OdbxBackend::getDomainInfo( const DNSName& domain, DomainInfo& di )
 
         		if( ( tmp = odbx_field_value( m_result, 3 ) ) != NULL )
         		{
-        			stringtok(di.masters, string( tmp, odbx_field_length( m_result, 3 ) ), ", \t");
+        			vector<string> masters;
+        			stringtok(masters, string( tmp, odbx_field_length( m_result, 3 ) ), ", \t");
+        			for(const auto& m : masters)
+        			{
+        				di.masters.emplace_back(m, 53);
+        			}
         		}
 
         		if( ( tmp = odbx_field_value( m_result, 2 ) ) != NULL )
@@ -170,7 +175,7 @@ bool OdbxBackend::getDomainInfo( const DNSName& domain, DomainInfo& di )
         }
         catch( std::exception& e )
         {
-        	L.log( m_myname + " getDomainInfo: Caught STL std::exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " getDomainInfo: Caught STL std::exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -179,14 +184,14 @@ bool OdbxBackend::getDomainInfo( const DNSName& domain, DomainInfo& di )
 
 
 
-bool OdbxBackend::getSOA( const DNSName& domain, SOAData& sd)
+bool OdbxBackend::getSOA( const DNSName& domain, SOAData& sd )
 {
         const char* tmp;
 
 
         try
         {
-        	DLOG( L.log( m_myname + " getSOA()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " getSOA()", Logger::Debug ) );
 
         	string stmt = getArg( "sql-lookupsoa" );
         	string& stmtref = strbind( ":name", escape( domain.makeLowerCase().toStringRootDot(), READ ), stmt );
@@ -236,7 +241,7 @@ bool OdbxBackend::getSOA( const DNSName& domain, SOAData& sd)
         }
         catch( std::exception& e )
         {
-        	L.log( m_myname + " getSOA: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " getSOA: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -249,22 +254,22 @@ bool OdbxBackend::list( const DNSName& target, int zoneid, bool include_disabled
 {
         try
         {
-        	DLOG( L.log( m_myname + " list()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " list()", Logger::Debug ) );
 
         	m_qname.clear();
         	m_result = NULL;
 
-        	int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, "%d", zoneid );
+        	int len = snprintf( m_buffer, sizeof( m_buffer ), "%d", zoneid );
 
         	if( len < 0 )
         	{
-        		L.log( m_myname + " list: Unable to convert zone id to string - format error",  Logger::Error );
+        		g_log.log( m_myname + " list: Unable to convert zone id to string - format error",  Logger::Error );
         		return false;
         	}
 
         	if( len > static_cast<int>(sizeof( m_buffer )) - 1 )
         	{
-        		L.log( m_myname + " list: Unable to convert zone id to string - insufficient buffer space",  Logger::Error );
+        		g_log.log( m_myname + " list: Unable to convert zone id to string - insufficient buffer space",  Logger::Error );
         		return false;
         	}
 
@@ -275,7 +280,7 @@ bool OdbxBackend::list( const DNSName& target, int zoneid, bool include_disabled
         }
         catch( std::exception& e )
         {
-        	L.log( m_myname + " list: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " list: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -288,7 +293,7 @@ void OdbxBackend::lookup( const QType& qtype, const DNSName& qname, DNSPacket* d
 {
         try
         {
-        	DLOG( L.log( m_myname + " lookup()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " lookup()", Logger::Debug ) );
 
         	string stmt;
         	string& stmtref = stmt;
@@ -316,17 +321,17 @@ void OdbxBackend::lookup( const QType& qtype, const DNSName& qname, DNSPacket* d
         			stmtref = strbind( ":type", qtype.getName(), stmt );
         		}
 
-        		int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, "%d", zoneid );
+        		int len = snprintf( m_buffer, sizeof( m_buffer ), "%d", zoneid );
 
         		if( len < 0 )
         		{
-        			L.log( m_myname + " lookup: Unable to convert zone id to string - format error",  Logger::Error );
+        			g_log.log( m_myname + " lookup: Unable to convert zone id to string - format error",  Logger::Error );
         			throw( DBException( "Error: Libc error" ) );
         		}
 
         		if( len > static_cast<int>(sizeof( m_buffer )) - 1 )
         		{
-        			L.log( m_myname + " lookup: Unable to convert zone id to string - insufficient buffer space",  Logger::Error );
+        			g_log.log( m_myname + " lookup: Unable to convert zone id to string - insufficient buffer space",  Logger::Error );
         			throw( DBException( "Error: Libc error" ) );
         	}
 
@@ -342,7 +347,7 @@ void OdbxBackend::lookup( const QType& qtype, const DNSName& qname, DNSPacket* d
         }
         catch( std::exception& e )
         {
-        	L.log( m_myname + " lookup: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " lookup: Caught STL exception - " + e.what(),  Logger::Error );
         	throw( DBException( "Error: STL exception" ) );
         }
 }
@@ -356,7 +361,7 @@ bool OdbxBackend::get( DNSResourceRecord& rr )
 
         try
         {
-        	DLOG( L.log( m_myname + " get()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " get()", Logger::Debug ) );
 
         	if( getRecord( READ ) )
         	{
@@ -405,7 +410,7 @@ bool OdbxBackend::get( DNSResourceRecord& rr )
         }
         catch( std::exception& e )
         {
-        	L.log( m_myname + " get: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " get: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -420,25 +425,25 @@ void OdbxBackend::setFresh( uint32_t domain_id )
 
         try
         {
-        	DLOG( L.log( m_myname + " setFresh()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " setFresh()", Logger::Debug ) );
 
         	if( !m_handle[WRITE] && !connectTo( m_hosts[WRITE], WRITE ) )
         	{
-        		L.log( m_myname + " setFresh: Master server is unreachable",  Logger::Error );
+        		g_log.log( m_myname + " setFresh: Master server is unreachable",  Logger::Error );
         		throw( DBException( "Error: Server unreachable" ) );
         	}
 
-        	len = snprintf( m_buffer, sizeof( m_buffer ) - 1, getArg( "sql-update-lastcheck" ).c_str(), time( 0 ), domain_id );
+        	len = snprintf( m_buffer, sizeof( m_buffer ), getArg( "sql-update-lastcheck" ).c_str(), time( 0 ), domain_id );
 
         	if( len < 0 )
         	{
-        		L.log( m_myname + " setFresh: Unable to insert values into statement '" + getArg( "sql-update-lastcheck" ) + "' - format error",  Logger::Error );
+        		g_log.log( m_myname + " setFresh: Unable to insert values into statement '" + getArg( "sql-update-lastcheck" ) + "' - format error",  Logger::Error );
         		throw( DBException( "Error: Libc error" ) );
         	}
 
         	if( len > static_cast<int>(sizeof( m_buffer )) - 1 )
         	{
-        		L.log( m_myname + " setFresh: Unable to insert values into statement '" + getArg( "sql-update-lastcheck" ) + "' - insufficient buffer space",  Logger::Error );
+        		g_log.log( m_myname + " setFresh: Unable to insert values into statement '" + getArg( "sql-update-lastcheck" ) + "' - insufficient buffer space",  Logger::Error );
         		throw( DBException( "Error: Libc error" ) );
         	}
 
@@ -449,7 +454,7 @@ void OdbxBackend::setFresh( uint32_t domain_id )
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " setFresh: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " setFresh: Caught STL exception - " + e.what(),  Logger::Error );
         	throw( DBException( "Error: STL exception" ) );
         }
 }
@@ -460,25 +465,25 @@ void OdbxBackend::setNotified( uint32_t domain_id, uint32_t serial )
 {
         try
         {
-        	DLOG( L.log( m_myname + " setNotified()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " setNotified()", Logger::Debug ) );
 
         	if( !m_handle[WRITE] && !connectTo( m_hosts[WRITE], WRITE ) )
         	{
-        		L.log( m_myname + " setFresh: Master server is unreachable",  Logger::Error );
+        		g_log.log( m_myname + " setFresh: Master server is unreachable",  Logger::Error );
         		throw( DBException( "Error: Server unreachable" ) );
         	}
 
-        	int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, getArg( "sql-update-serial" ).c_str(), serial, domain_id );
+        	int len = snprintf( m_buffer, sizeof( m_buffer ), getArg( "sql-update-serial" ).c_str(), serial, domain_id );
 
         	if( len < 0 )
         	{
-        		L.log( m_myname + " setNotified: Unable to insert values into statement '" + getArg( "sql-update-serial" ) + "' - format error",  Logger::Error );
+        		g_log.log( m_myname + " setNotified: Unable to insert values into statement '" + getArg( "sql-update-serial" ) + "' - format error",  Logger::Error );
         		throw( DBException( "Error: Libc error" ) );
         	}
 
         	if( len > static_cast<int>(sizeof( m_buffer )) - 1 )
         	{
-        		L.log( m_myname + " setNotified: Unable to insert values into statement '" + getArg( "sql-update-serial" ) + "' - insufficient buffer space",  Logger::Error );
+        		g_log.log( m_myname + " setNotified: Unable to insert values into statement '" + getArg( "sql-update-serial" ) + "' - insufficient buffer space",  Logger::Error );
         		throw( DBException( "Error: Libc error" ) );
         	}
 
@@ -489,58 +494,22 @@ void OdbxBackend::setNotified( uint32_t domain_id, uint32_t serial )
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " setNotified: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " setNotified: Caught STL exception - " + e.what(),  Logger::Error );
         	throw( DBException( "Error: STL exception" ) );
         }
 }
 
 
 
-bool OdbxBackend::isMaster( const DNSName& domain, const string& ip )
-{
-        try
-        {
-        	DLOG( L.log( m_myname + " isMaster()", Logger::Debug ) );
-
-        	string stmt = getArg( "sql-master" );
-        	string& stmtref = strbind( ":name", escape( domain.makeLowerCase().toStringRootDot(), READ ), stmt );
-
-        	if( !execStmt( stmtref.c_str(), stmtref.size(), READ ) ) { return false; }
-        	if( !getRecord( READ ) ) { return false; }
-
-        	do
-        	{
-        		if( odbx_field_value( m_result, 0 ) != NULL )
-        		{
-        			if( !strcmp( odbx_field_value( m_result, 0 ), ip.c_str() ) )
-        			{
-        				while( getRecord( READ ) );
-        				return true;
-        			}
-        		}
-        	}
-        	while( getRecord( READ ) );
-        }
-        catch ( std::exception& e )
-        {
-        	L.log( m_myname + " isMaster: Caught STL exception - " + e.what(),  Logger::Error );
-        	return false;
-        }
-
-        return false;
-}
-
-
-
 void OdbxBackend::getUnfreshSlaveInfos( vector<DomainInfo>* unfresh )
 {
         try
         {
-        	DLOG( L.log( m_myname + " getUnfreshSlaveInfos()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " getUnfreshSlaveInfos()", Logger::Debug ) );
 
         	if( unfresh == NULL )
         	{
-        		L.log( m_myname + " getUnfreshSlaveInfos: invalid parameter - NULL pointer",  Logger::Error );
+        		g_log.log( m_myname + " getUnfreshSlaveInfos: invalid parameter - NULL pointer",  Logger::Error );
         		return;
         	}
 
@@ -548,7 +517,7 @@ void OdbxBackend::getUnfreshSlaveInfos( vector<DomainInfo>* unfresh )
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " getUnfreshSlaveInfo: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " getUnfreshSlaveInfo: Caught STL exception - " + e.what(),  Logger::Error );
         }
 }
 
@@ -558,11 +527,11 @@ void OdbxBackend::getUpdatedMasters( vector<DomainInfo>* updated )
 {
         try
         {
-        	DLOG( L.log( m_myname + " getUpdatedMasters()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " getUpdatedMasters()", Logger::Debug ) );
 
         	if( updated == NULL )
         	{
-        		L.log( m_myname + " getUpdatedMasters: invalid parameter - NULL pointer",  Logger::Error );
+        		g_log.log( m_myname + " getUpdatedMasters: invalid parameter - NULL pointer",  Logger::Error );
         		return;
         	}
 
@@ -570,7 +539,7 @@ void OdbxBackend::getUpdatedMasters( vector<DomainInfo>* updated )
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " getUpdatedMasters: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " getUpdatedMasters: Caught STL exception - " + e.what(),  Logger::Error );
         }
 }
 
@@ -580,7 +549,7 @@ bool OdbxBackend::superMasterBackend( const string& ip, const DNSName& domain, c
 {
         try
         {
-        	DLOG( L.log( m_myname + " superMasterBackend()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " superMasterBackend()", Logger::Debug ) );
 
         	if( account != NULL && ddb != NULL )
         	{
@@ -611,7 +580,7 @@ bool OdbxBackend::superMasterBackend( const string& ip, const DNSName& domain, c
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " superMasterBackend: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " superMasterBackend: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -624,26 +593,26 @@ bool OdbxBackend::createSlaveDomain( const string& ip, const DNSName& domain, co
 {
         try
         {
-        	DLOG( L.log( m_myname + " createSlaveDomain()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " createSlaveDomain()", Logger::Debug ) );
 
         	if( !m_handle[WRITE] && !connectTo( m_hosts[WRITE], WRITE ) )
         	{
-        		L.log( m_myname + " createSlaveDomain: Master server is unreachable",  Logger::Error );
+        		g_log.log( m_myname + " createSlaveDomain: Master server is unreachable",  Logger::Error );
         		return false;
         	}
 
-        	int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, getArg( "sql-insert-slave" ).c_str(), escape( domain.makeLowerCase().toStringRootDot(), WRITE ).c_str(),
+        	int len = snprintf( m_buffer, sizeof( m_buffer ), getArg( "sql-insert-slave" ).c_str(), escape( domain.makeLowerCase().toStringRootDot(), WRITE ).c_str(),
         		escape( ip, WRITE ).c_str(), escape( account, WRITE ).c_str() );
 
         	if( len < 0 )
         	{
-        		L.log( m_myname + " createSlaveDomain: Unable to insert values in statement '" + getArg( "sql-insert-slave" ) + "' - format error",  Logger::Error );
+        		g_log.log( m_myname + " createSlaveDomain: Unable to insert values in statement '" + getArg( "sql-insert-slave" ) + "' - format error",  Logger::Error );
         		return false;
         	}
 
         	if( len > static_cast<int>(sizeof( m_buffer )) - 1 )
         	{
-        		L.log( m_myname + " createSlaveDomain: Unable to insert values in statement '" + getArg( "sql-insert-slave" ) + "' - insufficient buffer space",  Logger::Error );
+        		g_log.log( m_myname + " createSlaveDomain: Unable to insert values in statement '" + getArg( "sql-insert-slave" ) + "' - insufficient buffer space",  Logger::Error );
         		return false;
         	}
 
@@ -651,7 +620,7 @@ bool OdbxBackend::createSlaveDomain( const string& ip, const DNSName& domain, co
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " createSlaveDomain: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " createSlaveDomain: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -660,15 +629,15 @@ bool OdbxBackend::createSlaveDomain( const string& ip, const DNSName& domain, co
 
 
 
-bool OdbxBackend::feedRecord( const DNSResourceRecord& rr, const DNSName& ordername )
+bool OdbxBackend::feedRecord( const DNSResourceRecord& rr, const DNSName& ordername, bool ordernameIsNSEC3 )
 {
         try
         {
-        	DLOG( L.log( m_myname + " feedRecord()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " feedRecord()", Logger::Debug ) );
 
         	if( !m_handle[WRITE] && !connectTo( m_hosts[WRITE], WRITE ) )
         	{
-        		L.log( m_myname + " feedRecord: Master server is unreachable",  Logger::Error );
+        		g_log.log( m_myname + " feedRecord: Master server is unreachable",  Logger::Error );
         		return false;
         	}
 
@@ -683,19 +652,19 @@ bool OdbxBackend::feedRecord( const DNSResourceRecord& rr, const DNSName& ordern
         		trim_left(content);
         	}
 
-        	int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, getArg( "sql-insert-record" ).c_str(), rr.domain_id,
+        	int len = snprintf( m_buffer, sizeof( m_buffer ), getArg( "sql-insert-record" ).c_str(), rr.domain_id,
         		escape( rr.qname.makeLowerCase().toStringRootDot(), WRITE ).c_str(), rr.qtype.getName().c_str(), rr.ttl, priority,
         		escape( content, WRITE ).c_str() );
 
         	if( len < 0 )
         	{
-        		L.log( m_myname + " feedRecord: Unable to insert values in statement '" + getArg( "sql-insert-record" ) + "' - format error",  Logger::Error );
+        		g_log.log( m_myname + " feedRecord: Unable to insert values in statement '" + getArg( "sql-insert-record" ) + "' - format error",  Logger::Error );
         		return false;
         	}
 
         	if( len > static_cast<int>(sizeof( m_buffer )) - 1 )
         	{
-        		L.log( m_myname + " feedRecord: Unable to insert values in statement '" + getArg( "sql-insert-record" ) + "' - insufficient buffer space",  Logger::Error );
+        		g_log.log( m_myname + " feedRecord: Unable to insert values in statement '" + getArg( "sql-insert-record" ) + "' - insufficient buffer space",  Logger::Error );
         		return false;
         	}
 
@@ -703,7 +672,7 @@ bool OdbxBackend::feedRecord( const DNSResourceRecord& rr, const DNSName& ordern
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " feedRecord: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " feedRecord: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -716,27 +685,27 @@ bool OdbxBackend::startTransaction( const DNSName& domain, int zoneid )
 {
         try
         {
-        	DLOG( L.log( m_myname + " startTransaction()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " startTransaction()", Logger::Debug ) );
 
         	if( !m_handle[WRITE] && !connectTo( m_hosts[WRITE], WRITE ) )
         	{
-        		L.log( m_myname + " startTransaction: Master server is unreachable",  Logger::Error );
+        		g_log.log( m_myname + " startTransaction: Master server is unreachable",  Logger::Error );
         		return false;
         	}
 
         	string stmtref =  getArg( "sql-transactbegin" );
         	if( !execStmt( stmtref.c_str(), stmtref.size(), WRITE ) ) { return false; }
-        	int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, "%d", zoneid );
+        	int len = snprintf( m_buffer, sizeof( m_buffer ), "%d", zoneid );
 
         	if( len < 0 )
         	{
-        		L.log( m_myname + " startTransaction: Unable to convert zone id to string - format error",  Logger::Error );
+        		g_log.log( m_myname + " startTransaction: Unable to convert zone id to string - format error",  Logger::Error );
         		return false;
         	}
 
         	if( len > static_cast<int>(sizeof( m_buffer )) - 1 )
         	{
-        		L.log( m_myname + " startTransaction: Unable to convert zone id to string - insufficient buffer space",  Logger::Error );
+        		g_log.log( m_myname + " startTransaction: Unable to convert zone id to string - insufficient buffer space",  Logger::Error );
         		return false;
         	}
 
@@ -748,7 +717,7 @@ bool OdbxBackend::startTransaction( const DNSName& domain, int zoneid )
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " startTransaction: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " startTransaction: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -761,11 +730,11 @@ bool OdbxBackend::commitTransaction()
 {
         try
         {
-        	DLOG( L.log( m_myname + " commitTransaction()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " commitTransaction()", Logger::Debug ) );
 
         	if( !m_handle[WRITE] && !connectTo( m_hosts[WRITE], WRITE ) )
         	{
-        		L.log( m_myname + " commitTransaction: Master server is unreachable",  Logger::Error );
+        		g_log.log( m_myname + " commitTransaction: Master server is unreachable",  Logger::Error );
         		return false;
         	}
 
@@ -774,7 +743,7 @@ bool OdbxBackend::commitTransaction()
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " commitTransaction: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " commitTransaction: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
@@ -787,11 +756,11 @@ bool OdbxBackend::abortTransaction()
 {
         try
         {
-        	DLOG( L.log( m_myname + " abortTransaction()", Logger::Debug ) );
+        	DLOG( g_log.log( m_myname + " abortTransaction()", Logger::Debug ) );
 
         	if( !m_handle[WRITE] && !connectTo( m_hosts[WRITE], WRITE ) )
         	{
-        		L.log( m_myname + " abortTransaction: Master server is unreachable",  Logger::Error );
+        		g_log.log( m_myname + " abortTransaction: Master server is unreachable",  Logger::Error );
         		return false;
         	}
 
@@ -800,7 +769,7 @@ bool OdbxBackend::abortTransaction()
         }
         catch ( std::exception& e )
         {
-        	L.log( m_myname + " abortTransaction: Caught STL exception - " + e.what(),  Logger::Error );
+        	g_log.log( m_myname + " abortTransaction: Caught STL exception - " + e.what(),  Logger::Error );
         	return false;
         }
 
