@@ -50,7 +50,7 @@ bool Connector::recv(Json& value) {
        if (value["result"].is_bool() && boolFromJson(value, "result", false) == false)
          rv = false;
        for(const auto& message: value["log"].array_items())
-         L<<Logger::Info<<"[remotebackend]: "<< message.string_value() <<std::endl;
+         g_log<<Logger::Info<<"[remotebackend]: "<< message.string_value() <<std::endl;
        return rv;
     }
     return false;
@@ -71,20 +71,16 @@ RemoteBackend::RemoteBackend(const std::string &suffix)
       build();
 }
 
-RemoteBackend::~RemoteBackend() {
-    if (connector != NULL) {
- 	delete connector;
-     }
-}
+RemoteBackend::~RemoteBackend() { }
 
 bool RemoteBackend::send(Json& value) {
    try {
      return connector->send(value);
    } catch (PDNSException &ex) {
-     L<<Logger::Error<<"Exception caught when sending: "<<ex.reason<<std::endl;
+     g_log<<Logger::Error<<"Exception caught when sending: "<<ex.reason<<std::endl;
    }
 
-   delete this->connector;
+   this->connector.reset();
    build();
    return false;
 }
@@ -93,12 +89,12 @@ bool RemoteBackend::recv(Json& value) {
    try {
      return connector->recv(value);
    } catch (PDNSException &ex) {
-     L<<Logger::Error<<"Exception caught when receiving: "<<ex.reason<<std::endl;
+     g_log<<Logger::Error<<"Exception caught when receiving: "<<ex.reason<<std::endl;
    } catch (...) {
-     L<<Logger::Error<<"Exception caught when receiving"<<std::endl;;
+     g_log<<Logger::Error<<"Exception caught when receiving"<<std::endl;;
    }
 
-   delete this->connector;
+   this->connector.reset();
    build();
    return false;
 }
@@ -147,17 +143,17 @@ int RemoteBackend::build() {
 
       // connectors know what they are doing
       if (type == "unix") {
-        this->connector = new UnixsocketConnector(options);
+        this->connector = std::unique_ptr<Connector>(new UnixsocketConnector(options));
       } else if (type == "http") {
-        this->connector = new HTTPConnector(options);
+        this->connector = std::unique_ptr<Connector>(new HTTPConnector(options));
       } else if (type == "zeromq") {
 #ifdef REMOTEBACKEND_ZEROMQ
-        this->connector = new ZeroMQConnector(options);
+        this->connector = std::unique_ptr<Connector>(new ZeroMQConnector(options));
 #else
         throw PDNSException("Invalid connection string: zeromq connector support not enabled. Recompile with --enable-remotebackend-zeromq");
 #endif
       } else if (type == "pipe") {
-        this->connector = new PipeConnector(options);
+        this->connector = std::unique_ptr<Connector>(new PipeConnector(options));
       } else {
         throw PDNSException("Invalid connection string: unknown connector");
       }
@@ -557,7 +553,7 @@ void RemoteBackend::parseDomainInfo(const Json &obj, DomainInfo &di)
    di.id = intFromJson(obj, "id", -1);
    di.zone = DNSName(stringFromJson(obj, "zone"));
    for(const auto& master: obj["masters"].array_items())
-     di.masters.push_back(master.string_value());
+     di.masters.push_back(ComboAddress(master.string_value(), 53));
 
    di.notified_serial = static_cast<unsigned int>(doubleFromJson(obj, "notified_serial", -1));
    di.serial = static_cast<unsigned int>(obj["serial"].number_value());
@@ -577,7 +573,7 @@ void RemoteBackend::parseDomainInfo(const Json &obj, DomainInfo &di)
    di.backend = this;
 }
 
-bool RemoteBackend::getDomainInfo(const DNSName& domain, DomainInfo &di) {
+bool RemoteBackend::getDomainInfo(const DNSName& domain, DomainInfo &di, bool getSerial) {
    if (domain.empty()) return false;
    Json query = Json::object{
      { "method", "getDomainInfo" },
@@ -605,27 +601,10 @@ void RemoteBackend::setNotified(uint32_t id, uint32_t serial) {
 
    Json answer;
    if (this->send(query) == false || this->recv(answer) == false) {
-      L<<Logger::Error<<kBackendId<<" Failed to execute RPC for RemoteBackend::setNotified("<<id<<","<<serial<<")"<<endl;
+      g_log<<Logger::Error<<kBackendId<<" Failed to execute RPC for RemoteBackend::setNotified("<<id<<","<<serial<<")"<<endl;
    }
 }
 
-bool RemoteBackend::isMaster(const DNSName& name, const string &ip)
-{
-   Json query = Json::object{
-     { "method", "isMaster" },
-     { "parameters", Json::object {
-       { "name", name.toString() },
-       { "ip", ip }
-     }}
-   };
-
-   Json answer;
-   if (this->send(query) == false || this->recv(answer) == false)
-     return false;
-
-   return true;
-}
-
 bool RemoteBackend::superMasterBackend(const string &ip, const DNSName& domain, const vector<DNSResourceRecord>&nsset, string* nameserver, string *account, DNSBackend **ddb)
 {
    Json::array rrset;
@@ -716,7 +695,7 @@ bool RemoteBackend::replaceRRSet(uint32_t domain_id, const DNSName& qname, const
    return true;
 }
 
-bool RemoteBackend::feedRecord(const DNSResourceRecord &rr, const DNSName &ordername) {
+bool RemoteBackend::feedRecord(const DNSResourceRecord &rr, const DNSName &ordername, bool ordernameIsNSEC3) {
    Json query = Json::object{
      { "method", "feedRecord" },
      { "parameters", Json::object{
@@ -845,35 +824,6 @@ bool RemoteBackend::abortTransaction() {
    return true;
 }
 
-bool RemoteBackend::calculateSOASerial(const DNSName& domain, const SOAData& sd, time_t& serial) {
-   Json query = Json::object{
-     { "method", "calculateSOASerial" },
-     { "parameters", Json::object{
-       { "domain", domain.toString() },
-       { "sd", Json::object{
-         { "qname", sd.qname.toString() },
-         { "nameserver", sd.nameserver.toString() },
-         { "hostmaster", sd.hostmaster.toString() },
-         { "ttl", static_cast<int>(sd.ttl) },
-         { "serial", static_cast<double>(sd.serial) },
-         { "refresh", static_cast<int>(sd.refresh) },
-         { "retry", static_cast<int>(sd.retry) },
-         { "expire", static_cast<int>(sd.expire) },
-         { "default_ttl", static_cast<int>(sd.default_ttl) },
-         { "domain_id", static_cast<int>(sd.domain_id) },
-         { "scopeMask", sd.scopeMask }
-       }}
-     }}
-   };
-
-   Json answer;
-   if (this->send(query) == false || this->recv(answer) == false)
-     return false;
-
-   serial = static_cast<unsigned int>(doubleFromJson(answer,"result"));
-   return true;
-}
-
 string RemoteBackend::directBackendCmd(const string& querystr) {
    Json query = Json::object{
      { "method", "directBackendCmd" },
@@ -954,13 +904,34 @@ void RemoteBackend::getAllDomains(vector<DomainInfo> *domains, bool include_disa
   }
 }
 
+void RemoteBackend::getUpdatedMasters(vector<DomainInfo>* domains)
+{
+  Json query = Json::object{
+   { "method", "getUpdatedMasters" },
+   { "parameters", Json::object{ } },
+  };
+
+  Json answer;
+  if (this->send(query) == false || this->recv(answer) == false)
+    return;
+
+  if (answer["result"].is_array() == false)
+    return;
+
+  for(const auto& row: answer["result"].array_items()) {
+    DomainInfo di;
+    this->parseDomainInfo(row, di);
+    domains->push_back(di);
+  }
+}
+
 DNSBackend *RemoteBackend::maker()
 {
    try {
       return new RemoteBackend();
    }
    catch(...) {
-      L<<Logger::Error<<kBackendId<<" Unable to instantiate a remotebackend!"<<endl;
+      g_log<<Logger::Error<<kBackendId<<" Unable to instantiate a remotebackend!"<<endl;
       return 0;
    };
 }
@@ -993,7 +964,7 @@ public:
 
 RemoteLoader::RemoteLoader() {
     BackendMakers().report(new RemoteBackendFactory);
-    L << Logger::Info << kBackendId << " This is the remote backend version " VERSION
+    g_log << Logger::Info << kBackendId << " This is the remote backend version " VERSION
 #ifndef REPRODUCIBLE
       << " (" __DATE__ " " __TIME__ ")"
 #endif
