@@ -24,6 +24,7 @@
 #endif
 #include "statbag.hh"
 #include "logger.hh"
+#include "threadname.hh"
 #include "iputils.hh"
 #include "sstuff.hh"
 #include "arguments.hh"
@@ -34,8 +35,10 @@
 void* carbonDumpThread(void*)
 try
 {
+  setThreadName("pdns/carbonDump");
   extern StatBag S;
 
+  string namespace_name=arg()["carbon-namespace"];
   string hostname=arg()["carbon-ourname"];
   if(hostname.empty()) {
     char tmp[80];
@@ -46,6 +49,7 @@ try
     hostname=tmp;
     boost::replace_all(hostname, ".", "_");
   }
+  string instance_name=arg()["carbon-instance"];
 
   vector<string> carbonServers;
   stringtok(carbonServers, arg()["carbon-server"], ", ");
@@ -61,7 +65,7 @@ try
     ostringstream str;
     time_t now=time(0);
     for(const string& entry : entries) {
-      str<<"pdns."<<hostname<<".auth."<<entry<<' '<<S.read(entry)<<' '<<now<<"\r\n";
+      str<<namespace_name<<'.'<<hostname<<'.'<<instance_name<<'.'<<entry<<' '<<S.read(entry)<<' '<<now<<"\r\n";
     }
     msg = str.str();
 
@@ -75,7 +79,7 @@ try
 
         writen2WithTimeout(s.getHandle(), msg.c_str(), msg.length(), 2);
       } catch (runtime_error &e){
-        L<<Logger::Warning<<"Unable to write data to carbon server at "<<remote.toStringWithPort()<<": "<<e.what()<<endl;
+        g_log<<Logger::Warning<<"Unable to write data to carbon server at "<<remote.toStringWithPort()<<": "<<e.what()<<endl;
         continue;
       }
     }
@@ -85,16 +89,16 @@ try
 }
 catch(std::exception& e)
 {
-  L<<Logger::Error<<"Carbon thread died: "<<e.what()<<endl;
+  g_log<<Logger::Error<<"Carbon thread died: "<<e.what()<<endl;
   return 0;
 }
 catch(PDNSException& e)
 {
-  L<<Logger::Error<<"Carbon thread died, PDNSException: "<<e.reason<<endl;
+  g_log<<Logger::Error<<"Carbon thread died, PDNSException: "<<e.reason<<endl;
   return 0;
 }
 catch(...)
 {
-  L<<Logger::Error<<"Carbon thread died"<<endl;
+  g_log<<Logger::Error<<"Carbon thread died"<<endl;
   return 0;
 }
