@@ -65,13 +65,12 @@ AuthPacketCache::~AuthPacketCache()
 
 bool AuthPacketCache::get(DNSPacket *p, DNSPacket *cached)
 {
-  cleanupIfNeeded();
-
   if(!d_ttl) {
-    (*d_statnummiss)++;
     return false;
   }
 
+  cleanupIfNeeded();
+
   uint32_t hash = canHashPacket(p->getString());
   p->setHash(hash);
 
@@ -113,6 +112,10 @@ bool AuthPacketCache::entryMatches(cmap_t::index<HashTag>::type::iterator& iter,
 
 void AuthPacketCache::insert(DNSPacket *q, DNSPacket *r, unsigned int maxTTL)
 {
+  if(!d_ttl) {
+    return;
+  }
+
   cleanupIfNeeded();
 
   if (ntohs(q->d.qdcount) != 1) {
@@ -192,6 +195,10 @@ bool AuthPacketCache::getEntryLocked(cmap_t& map, const std::string& query, uint
 /* clears the entire cache. */
 uint64_t AuthPacketCache::purge()
 {
+  if(!d_ttl) {
+    return 0;
+  }
+
   d_statnumentries->store(0);
 
   return purgeLockedCollectionsVector(d_maps);
@@ -210,6 +217,10 @@ uint64_t AuthPacketCache::purgeExact(const DNSName& qname)
 /* purges entries from the packetcache. If match ends on a $, it is treated as a suffix */
 uint64_t AuthPacketCache::purge(const string &match)
 {
+  if(!d_ttl) {
+    return 0;
+  }
+
   uint64_t delcount = 0;
 
   if(ends_with(match, "$")) {
@@ -232,7 +243,7 @@ void AuthPacketCache::cleanup()
   totErased = pruneLockedCollectionsVector(d_maps, maxCached, cacheSize);
   *d_statnumentries -= totErased;
 
-  DLOG(L<<"Done with cache clean, cacheSize: "<<(*d_statnumentries)<<", totErased"<<totErased<<endl);
+  DLOG(g_log<<"Done with cache clean, cacheSize: "<<(*d_statnumentries)<<", totErased"<<totErased<<endl);
 }
 
 /* the logic:
@@ -250,13 +261,13 @@ void AuthPacketCache::cleanupIfNeeded()
     time_t now = time(nullptr);
     int timediff = max((int)(now - d_lastclean), 1);
 
-    DLOG(L<<"cleaninterval: "<<d_cleaninterval<<", timediff: "<<timediff<<endl);
+    DLOG(g_log<<"cleaninterval: "<<d_cleaninterval<<", timediff: "<<timediff<<endl);
 
     if (d_cleaninterval == s_maxcleaninterval && timediff < 30) {
       d_cleanskipped = true;
       d_nextclean += d_cleaninterval;
 
-      DLOG(L<<"cleaning skipped, timediff: "<<timediff<<endl);
+      DLOG(g_log<<"cleaning skipped, timediff: "<<timediff<<endl);
 
       return;
     }
@@ -266,7 +277,7 @@ void AuthPacketCache::cleanupIfNeeded()
       d_cleaninterval=std::max(d_cleaninterval, s_mincleaninterval);
       d_cleaninterval=std::min(d_cleaninterval, s_maxcleaninterval);
 
-      DLOG(L<<"new cleaninterval: "<<d_cleaninterval<<endl);
+      DLOG(g_log<<"new cleaninterval: "<<d_cleaninterval<<endl);
     } else {
       d_cleanskipped = false;
     }
