@@ -205,7 +205,7 @@ void AuthQueryCache::cleanup()
   totErased = pruneLockedCollectionsVector(d_maps, maxCached, cacheSize);
 
   *d_statnumentries -= totErased;
-  DLOG(L<<"Done with cache clean, cacheSize: "<<*d_statnumentries<<", totErased"<<totErased<<endl);
+  DLOG(g_log<<"Done with cache clean, cacheSize: "<<*d_statnumentries<<", totErased"<<totErased<<endl);
 }
 
 /* the logic:
@@ -223,13 +223,13 @@ void AuthQueryCache::cleanupIfNeeded()
     time_t now = time(nullptr);
     int timediff = max((int)(now - d_lastclean), 1);
 
-    DLOG(L<<"cleaninterval: "<<d_cleaninterval<<", timediff: "<<timediff<<endl);
+    DLOG(g_log<<"cleaninterval: "<<d_cleaninterval<<", timediff: "<<timediff<<endl);
 
     if (d_cleaninterval == s_maxcleaninterval && timediff < 30) {
       d_cleanskipped = true;
       d_nextclean += d_cleaninterval;
 
-      DLOG(L<<"cleaning skipped, timediff: "<<timediff<<endl);
+      DLOG(g_log<<"cleaning skipped, timediff: "<<timediff<<endl);
 
       return;
     }
@@ -239,7 +239,7 @@ void AuthQueryCache::cleanupIfNeeded()
       d_cleaninterval=std::max(d_cleaninterval, s_mincleaninterval);
       d_cleaninterval=std::min(d_cleaninterval, s_maxcleaninterval);
 
-      DLOG(L<<"new cleaninterval: "<<d_cleaninterval<<endl);
+      DLOG(g_log<<"new cleaninterval: "<<d_cleaninterval<<endl);
     } else {
       d_cleanskipped = false;
     }
