@@ -36,10 +36,12 @@
 #include "dns.hh"
 #include "arguments.hh"
 #include "packetcache.hh"
+#include "threadname.hh"
 
 // there can be MANY OF THESE
 void CommunicatorClass::retrievalLoopThread(void)
 {
+  setThreadName("pdns/comm-retre");
   for(;;) {
     d_suck_sem.wait();
     SuckRequest sr;
@@ -65,7 +67,7 @@ void CommunicatorClass::loadArgsIntoSet(const char *listname, set<string> &lists
       listset.insert(caIp.toStringWithPort());
     }
     catch(PDNSException &e) {
-      L<<Logger::Error<<"Unparseable IP in "<<listname<<". Error: "<<e.reason<<endl;
+      g_log<<Logger::Error<<"Unparseable IP in "<<listname<<". Error: "<<e.reason<<endl;
       _exit(1);
     }
   }
@@ -77,7 +79,7 @@ void CommunicatorClass::go()
     PacketHandler::s_allowNotifyFrom.toMasks(::arg()["allow-notify-from"] );
   }
   catch(PDNSException &e) {
-    L<<Logger::Error<<"Unparseable IP in allow-notify-from. Error: "<<e.reason<<endl;
+    g_log<<Logger::Error<<"Unparseable IP in allow-notify-from. Error: "<<e.reason<<endl;
     _exit(1);
   }
 
@@ -92,7 +94,7 @@ void CommunicatorClass::go()
     d_onlyNotify.toMasks(::arg()["only-notify"]);
   }
   catch(PDNSException &e) {
-    L<<Logger::Error<<"Unparseable IP in only-notify. Error: "<<e.reason<<endl;
+    g_log<<Logger::Error<<"Unparseable IP in only-notify. Error: "<<e.reason<<endl;
     _exit(1);
   }
 
@@ -104,8 +106,9 @@ void CommunicatorClass::go()
 void CommunicatorClass::mainloop(void)
 {
   try {
+    setThreadName("pdns/comm-main");
     signal(SIGPIPE,SIG_IGN);
-    L<<Logger::Error<<"Master/slave communicator launching"<<endl;
+    g_log<<Logger::Error<<"Master/slave communicator launching"<<endl;
     PacketHandler P;
     d_tickinterval=::arg().asNum("slave-cycle-interval");
     makeNotifySockets();
@@ -116,7 +119,7 @@ void CommunicatorClass::mainloop(void)
     for(;;) {
       slaveRefresh(&P);
       masterUpdateCheck(&P);
-      tick=doNotifications(); // this processes any notification acknowledgements and actually send out our own notifications
+      tick=doNotifications(&P); // this processes any notification acknowledgements and actually send out our own notifications
       
       tick = min (tick, d_tickinterval); 
       
@@ -136,26 +139,29 @@ void CommunicatorClass::mainloop(void)
           if (extraSlaveRefresh)
             slaveRefresh(&P);
         }
-        else { 
+        else {
+          // eat up extra posts to avoid busy looping if many posts were done
+          while (d_any_sem.tryWait() == 0) {
+          }
           break; // something happened
         }
         // this gets executed at least once every second
-        doNotifications();
+        doNotifications(&P);
       }
     }
   }
   catch(PDNSException &ae) {
-    L<<Logger::Error<<"Exiting because communicator thread died with error: "<<ae.reason<<endl;
+    g_log<<Logger::Error<<"Exiting because communicator thread died with error: "<<ae.reason<<endl;
     Utility::sleep(1);
     _exit(1);
   }
   catch(std::exception &e) {
-    L<<Logger::Error<<"Exiting because communicator thread died with STL error: "<<e.what()<<endl;
+    g_log<<Logger::Error<<"Exiting because communicator thread died with STL error: "<<e.what()<<endl;
     _exit(1);
   }
   catch( ... )
   {
-    L << Logger::Error << "Exiting because communicator caught unknown exception." << endl;
+    g_log << Logger::Error << "Exiting because communicator caught unknown exception." << endl;
     _exit(1);
   }
 }
