@@ -30,12 +30,15 @@
 #include <string>
 #include <errno.h>
 #include <boost/tokenizer.hpp>
+#include <boost/functional/hash.hpp>
 #include <boost/algorithm/string.hpp>
+#include <openssl/hmac.h>
 #include <algorithm>
 
 #include "dnsseckeeper.hh"
 #include "dns.hh"
 #include "dnsbackend.hh"
+#include "ednsoptions.hh"
 #include "pdnsexception.hh"
 #include "dnspacket.hh"
 #include "logger.hh"
@@ -43,7 +46,7 @@
 #include "dnswriter.hh"
 #include "dnsparser.hh"
 #include "dnsrecords.hh"
-#include "dnssecinfra.hh" 
+#include "dnssecinfra.hh"
 #include "base64.hh"
 #include "ednssubnet.hh"
 #include "gss_context.hh"
@@ -52,25 +55,9 @@
 bool DNSPacket::s_doEDNSSubnetProcessing;
 uint16_t DNSPacket::s_udpTruncationThreshold;
  
-DNSPacket::DNSPacket(bool isQuery)
+DNSPacket::DNSPacket(bool isQuery): d_isQuery(isQuery)
 {
-  d_wrapped=false;
-  d_compress=true;
-  d_tcp=false;
-  d_wantsnsid=false;
-  d_haveednssubnet = false;
-  d_dnssecOk=false;
-  d_ednsversion=0;
-  d_ednsrcode=0;
   memset(&d, 0, sizeof(d));
-  qclass = QClass::IN;
-  d_tsig_algo = TSIG_MD5;
-  d_havetsig = false;
-  d_socket = -1;
-  d_maxreplylen = 0;
-  d_tsigtimersonly = false;
-  d_haveednssection = false;
-  d_isQuery = isQuery;
 }
 
 const string& DNSPacket::getString()
@@ -91,46 +78,47 @@ uint16_t DNSPacket::getRemotePort() const
   return d_remote.sin4.sin_port;
 }
 
-DNSPacket::DNSPacket(const DNSPacket &orig)
+DNSPacket::DNSPacket(const DNSPacket &orig) :
+  d_anyLocal(orig.d_anyLocal),
+  d_dt(orig.d_dt),
+  qdomain(orig.qdomain),
+  qdomainwild(orig.qdomainwild),
+  qdomainzone(orig.qdomainzone),
+
+  d(orig.d),
+  d_trc(orig.d_trc),
+  d_remote(orig.d_remote),
+  d_tsig_algo(orig.d_tsig_algo),
+
+  d_ednsRawPacketSizeLimit(orig.d_ednsRawPacketSizeLimit),
+  qclass(orig.qclass),
+  qtype(orig.qtype),
+  d_tcp(orig.d_tcp),
+  d_dnssecOk(orig.d_dnssecOk),
+  d_havetsig(orig.d_havetsig),
+
+  d_tsigsecret(orig.d_tsigsecret),
+  d_tsigkeyname(orig.d_tsigkeyname),
+  d_tsigprevious(orig.d_tsigprevious),
+  d_rrs(orig.d_rrs),
+  d_rawpacket(orig.d_rawpacket),
+  d_eso(orig.d_eso),
+  d_maxreplylen(orig.d_maxreplylen),
+  d_socket(orig.d_socket),
+  d_hash(orig.d_hash),
+  d_ednsrcode(orig.d_ednsrcode),
+  d_ednsversion(orig.d_ednsversion),
+
+  d_wrapped(orig.d_wrapped),
+  d_compress(orig.d_compress),
+  d_tsigtimersonly(orig.d_tsigtimersonly),
+  d_wantsnsid(orig.d_wantsnsid),
+  d_haveednssubnet(orig.d_haveednssubnet),
+  d_haveednssection(orig.d_haveednssection),
+
+  d_isQuery(orig.d_isQuery)
 {
-  DLOG(L<<"DNSPacket copy constructor called!"<<endl);
-  d_socket=orig.d_socket;
-  d_remote=orig.d_remote;
-  d_dt=orig.d_dt;
-  d_compress=orig.d_compress;
-  d_tcp=orig.d_tcp;
-  qtype=orig.qtype;
-  qclass=orig.qclass;
-  qdomain=orig.qdomain;
-  qdomainwild=orig.qdomainwild;
-  qdomainzone=orig.qdomainzone;
-  d_maxreplylen = orig.d_maxreplylen;
-  d_ednsping = orig.d_ednsping;
-  d_wantsnsid = orig.d_wantsnsid;
-  d_anyLocal = orig.d_anyLocal;  
-  d_eso = orig.d_eso;
-  d_haveednssubnet = orig.d_haveednssubnet;
-  d_haveednssection = orig.d_haveednssection;
-  d_ednsversion = orig.d_ednsversion;
-  d_ednsrcode = orig.d_ednsrcode;
-  d_dnssecOk = orig.d_dnssecOk;
-  d_rrs=orig.d_rrs;
-  
-  d_tsigkeyname = orig.d_tsigkeyname;
-  d_tsigprevious = orig.d_tsigprevious;
-  d_tsigtimersonly = orig.d_tsigtimersonly;
-  d_trc = orig.d_trc;
-  d_tsigsecret = orig.d_tsigsecret;
-  
-  d_havetsig = orig.d_havetsig;
-  d_wrapped=orig.d_wrapped;
-
-  d_rawpacket=orig.d_rawpacket;
-  d_tsig_algo=orig.d_tsig_algo;
-  d=orig.d;
-
-  d_isQuery = orig.d_isQuery;
-  d_hash = orig.d_hash;
+  DLOG(g_log<<"DNSPacket copy constructor called!"<<endl);
 }
 
 void DNSPacket::setRcode(int v)
@@ -173,30 +161,32 @@ void DNSPacket::setOpcode(uint16_t opcode)
   d.opcode=opcode;
 }
 
-
 void DNSPacket::clearRecords()
 {
   d_rrs.clear();
+  d_dedup.clear();
 }
 
 void DNSPacket::addRecord(const DNSZoneRecord &rr)
 {
-  // this removes duplicates from the packet in case we are not compressing
-  // for AXFR, no such checking is performed!
-  // cerr<<"addrecord, content=["<<rr.content<<"]"<<endl;
+  // this removes duplicates from the packet.
+  // in case we are not compressing for AXFR, no such checking is performed!
+
   if(d_compress) {
-    for(auto i=d_rrs.begin();i!=d_rrs.end();++i) {
-      if(rr.dr == i->dr)  // XXX SUPER SLOW
+    std::string ser = const_cast<DNSZoneRecord&>(rr).dr.d_content->serialize(rr.dr.d_name);
+    auto hash = boost::hash< std::pair<DNSName, std::string> >()({rr.dr.d_name, ser});
+    if(d_dedup.count(hash)) { // might be a dup
+      for(auto i=d_rrs.begin();i!=d_rrs.end();++i) {
+        if(rr.dr == i->dr)  // XXX SUPER SLOW
           return;
+      }
     }
+    d_dedup.insert(hash);
   }
 
-  // cerr<<"added to d_rrs"<<endl;
   d_rrs.push_back(rr);
 }
 
-
-
 vector<DNSZoneRecord*> DNSPacket::getAPRecords()
 {
   vector<DNSZoneRecord*> arrs;
@@ -213,9 +203,7 @@ vector<DNSZoneRecord*> DNSPacket::getAPRecords()
           arrs.push_back(&*i);
         }
     }
-
   return arrs;
-
 }
 
 vector<DNSZoneRecord*> DNSPacket::getAnswerRecords()
@@ -242,7 +230,7 @@ void DNSPacket::setCompress(bool compress)
 
 bool DNSPacket::couldBeCached()
 {
-  return d_ednsping.empty() && !d_wantsnsid && qclass==QClass::IN && !d_havetsig;
+  return !d_wantsnsid && qclass==QClass::IN && !d_havetsig;
 }
 
 unsigned int DNSPacket::getMinTTL()
@@ -299,18 +287,48 @@ void DNSPacket::wrapup()
   pw.getHeader()->tc=d.tc;
   
   DNSPacketWriter::optvect_t opts;
+
+  /* optsize is expected to hold an upper bound of data that will be
+     added after actual record data - i.e. OPT, TSIG, perhaps one day
+     XPF. Because of the way `pw` incrementally writes the packet, we
+     cannot easily 'go back' and remove a few records. So, to prevent
+     going over our maximum size, we keep our (potential) extra data
+     in mind.
+
+     This means that sometimes we'll send TC even if we'd end up with
+     a few bytes to spare, but so be it.
+    */
+  size_t optsize = 0;
+
+  if (d_haveednssection || d_dnssecOk) {
+    /* root label (1), type (2), class (2), ttl (4) + rdlen (2) */
+    optsize = 11;
+  }
+
   if(d_wantsnsid) {
     const static string mode_server_id=::arg()["server-id"];
     if(mode_server_id != "disabled") {
-      opts.push_back(make_pair(3, mode_server_id));
+      opts.push_back(make_pair(EDNSOptionCode::NSID, mode_server_id));
+      optsize += EDNS_OPTION_CODE_SIZE + EDNS_OPTION_LENGTH_SIZE + mode_server_id.size();
     }
   }
 
-  if(!d_ednsping.empty()) {
-    opts.push_back(make_pair(4, d_ednsping));
+  if (d_haveednssubnet)
+  {
+    // this is an upper bound
+    optsize += EDNS_OPTION_CODE_SIZE + EDNS_OPTION_LENGTH_SIZE + 2 + 1 + 1; // code+len+family+src len+scope len
+    optsize += d_eso.source.isIpv4() ? 4 : 16;
   }
-  
-  
+
+  if (d_trc.d_algoName.countLabels())
+  {
+    // TSIG is not OPT, but we count it in optsize anyway
+    optsize += d_trc.d_algoName.wirelength() + 3 + 1 + 2; // algo + time + fudge + maclen
+    optsize += EVP_MAX_MD_SIZE + 2 + 2 + 2 + 0; // mac + origid + ercode + otherdatalen + no other data
+
+    static_assert(EVP_MAX_MD_SIZE <= 64, "EVP_MAX_MD_SIZE is overly huge on this system, please check");
+  }
+
   if(!d_rrs.empty() || !opts.empty() || d_haveednssubnet || d_haveednssection) {
     try {
       uint8_t maxScopeMask=0;
@@ -320,12 +338,10 @@ void DNSPacket::wrapup()
         
         pw.startRecord(pos->dr.d_name, pos->dr.d_type, pos->dr.d_ttl, pos->dr.d_class, pos->dr.d_place);
         pos->dr.d_content->toPacket(pw);
-        if(pw.size() + 20U > (d_tcp ? 65535 : getMaxReplyLen())) { // 20 = room for EDNS0
+        if(pw.size() + optsize > (d_tcp ? 65535 : getMaxReplyLen())) {
           pw.rollback();
-          if(pos->dr.d_place == DNSResourceRecord::ANSWER || pos->dr.d_place == DNSResourceRecord::AUTHORITY) {
-            pw.truncate();
-            pw.getHeader()->tc=1;
-          }
+          pw.truncate();
+          pw.getHeader()->tc=1;
           goto noCommit;
         }
       }
@@ -337,7 +353,6 @@ void DNSPacket::wrapup()
       noCommit:;
       
       if(d_haveednssubnet) {
-        string makeEDNSSubnetOptsString(const EDNSSubnetOpts& eso);
         EDNSSubnetOpts eso = d_eso;
         eso.scope = Netmask(eso.source.getNetwork(), maxScopeMask);
     
@@ -352,7 +367,7 @@ void DNSPacket::wrapup()
       }
     }
     catch(std::exception& e) {
-      L<<Logger::Warning<<"Exception: "<<e.what()<<endl;
+      g_log<<Logger::Warning<<"Exception: "<<e.what()<<endl;
       throw;
     }
   }
@@ -403,7 +418,6 @@ DNSPacket *DNSPacket::replyPacket() const
   r->qtype = qtype;
   r->qclass = qclass;
   r->d_maxreplylen = d_maxreplylen;
-  r->d_ednsping = d_ednsping;
   r->d_wantsnsid = d_wantsnsid;
   r->d_dnssecOk = d_dnssecOk;
   r->d_eso = d_eso;
@@ -443,12 +457,11 @@ int DNSPacket::noparse(const char *mesg, size_t length)
 {
   d_rawpacket.assign(mesg,length); 
   if(length < 12) { 
-    L << Logger::Debug << "Ignoring packet: too short ("<<length<<" < 12) from "
+    g_log << Logger::Debug << "Ignoring packet: too short ("<<length<<" < 12) from "
       << d_remote.toStringWithPort()<< endl;
     return -1;
   }
   d_wantsnsid=false;
-  d_ednsping.clear();
   d_maxreplylen=512;
   memcpy((void *)&d,(const void *)d_rawpacket.c_str(),12);
   return 0;
@@ -477,7 +490,7 @@ bool DNSPacket::getTSIGDetails(TSIGRecordContent* trc, DNSName* keyname, uint16_
       // cast can fail, f.e. if d_content is an UnknownRecordContent.
       shared_ptr<TSIGRecordContent> content = std::dynamic_pointer_cast<TSIGRecordContent>(i->first.d_content);
       if (!content) {
-        L<<Logger::Error<<"TSIG record has no or invalid content (invalid packet)"<<endl;
+        g_log<<Logger::Error<<"TSIG record has no or invalid content (invalid packet)"<<endl;
         return false;
       }
       *trc = *content;
@@ -502,7 +515,7 @@ bool DNSPacket::getTKEYRecord(TKEYRecordContent *tr, DNSName *keyname) const
 
   for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {
     if (gotit) {
-      L<<Logger::Error<<"More than one TKEY record found in query"<<endl;
+      g_log<<Logger::Error<<"More than one TKEY record found in query"<<endl;
       return false;
     }
 
@@ -510,7 +523,7 @@ bool DNSPacket::getTKEYRecord(TKEYRecordContent *tr, DNSName *keyname) const
       // cast can fail, f.e. if d_content is an UnknownRecordContent.
       shared_ptr<TKEYRecordContent> content = std::dynamic_pointer_cast<TKEYRecordContent>(i->first.d_content);
       if (!content) {
-        L<<Logger::Error<<"TKEY record has no or invalid content (invalid packet)"<<endl;
+        g_log<<Logger::Error<<"TKEY record has no or invalid content (invalid packet)"<<endl;
         return false;
       }
       *tr = *content;
@@ -532,7 +545,7 @@ try
   d_rawpacket.assign(mesg,length); 
   d_wrapped=true;
   if(length < 12) { 
-    L << Logger::Debug << "Ignoring packet: too short from "
+    g_log << Logger::Debug << "Ignoring packet: too short from "
       << getRemote() << endl;
     return -1;
   }
@@ -544,32 +557,28 @@ try
 
   d_wantsnsid=false;
   d_dnssecOk=false;
-  d_ednsping.clear();
   d_havetsig = mdp.getTSIGPos();
   d_haveednssubnet = false;
   d_haveednssection = false;
-  
 
   if(getEDNSOpts(mdp, &edo)) {
     d_haveednssection=true;
     /* rfc6891 6.2.3:
        "Values lower than 512 MUST be treated as equal to 512."
     */
+    d_ednsRawPacketSizeLimit=edo.d_packetsize;
     d_maxreplylen=std::min(std::max(static_cast<uint16_t>(512), edo.d_packetsize), s_udpTruncationThreshold);
-//    cerr<<edo.d_Z<<endl;
-    if(edo.d_Z & EDNSOpts::DNSSECOK)
+//    cerr<<edo.d_extFlags<<endl;
+    if(edo.d_extFlags & EDNSOpts::DNSSECOK)
       d_dnssecOk=true;
 
     for(vector<pair<uint16_t, string> >::const_iterator iter = edo.d_options.begin();
         iter != edo.d_options.end(); 
         ++iter) {
-      if(iter->first == 3) {// 'EDNS NSID'
-        d_wantsnsid=1;
+      if(iter->first == EDNSOptionCode::NSID) {
+        d_wantsnsid=true;
       }
-      else if(iter->first == 5) {// 'EDNS PING'
-        d_ednsping = iter->second;
-      }
-      else if(s_doEDNSSubnetProcessing && (iter->first == 8)) { // 'EDNS SUBNET'
+      else if(s_doEDNSSubnetProcessing && (iter->first == EDNSOptionCode::ECS)) { // 'EDNS SUBNET'
         if(getEDNSSubnetOptsFromString(iter->second, &d_eso)) {
           //cerr<<"Parsed, source: "<<d_eso.source.toString()<<", scope: "<<d_eso.scope.toString()<<", family = "<<d_eso.scope.getNetwork().sin4.sin_family<<endl;
           d_haveednssubnet=true;
@@ -581,9 +590,10 @@ try
     }
     d_ednsversion = edo.d_version;
     d_ednsrcode = edo.d_extRCode;
-  }
+ }
   else  {
     d_maxreplylen=512;
+    d_ednsRawPacketSizeLimit=-1;
   }
 
   memcpy((void *)&d,(const void *)d_rawpacket.c_str(),12);
@@ -593,7 +603,7 @@ try
 
   if(!ntohs(d.qdcount)) {
     if(!d_tcp) {
-      L << Logger::Warning << "No question section in packet from " << getRemote() <<", error="<<RCode::to_s(d.rcode)<<endl;
+      g_log << Logger::Warning << "No question section in packet from " << getRemote() <<", error="<<RCode::to_s(d.rcode)<<endl;
       return -1;
     }
   }
@@ -625,7 +635,7 @@ void DNSPacket::setRemote(const ComboAddress *s)
   d_remote=*s;
 }
 
-bool DNSPacket::hasEDNSSubnet()
+bool DNSPacket::hasEDNSSubnet() const
 {
   return d_haveednssubnet;
 }
@@ -669,7 +679,7 @@ bool DNSPacket::checkForCorrectTSIG(UeberBackend* B, DNSName* keyname, string* s
   string secret64;
   if (tt.algo != DNSName("gss-tsig")) {
     if(!B->getTSIGKey(*keyname, &tt.algo, &secret64)) {
-      L<<Logger::Error<<"Packet for domain '"<<this->qdomain<<"' denied: can't find TSIG key with name '"<<*keyname<<"' and algorithm '"<<tt.algo<<"'"<<endl;
+      g_log<<Logger::Error<<"Packet for domain '"<<this->qdomain<<"' denied: can't find TSIG key with name '"<<*keyname<<"' and algorithm '"<<tt.algo<<"'"<<endl;
       return false;
     }
     B64Decode(secret64, *secret);
@@ -682,7 +692,7 @@ bool DNSPacket::checkForCorrectTSIG(UeberBackend* B, DNSName* keyname, string* s
     result = validateTSIG(d_rawpacket, tsigPos, tt, *trc, "", trc->d_mac, false);
   }
   catch(const std::runtime_error& err) {
-    L<<Logger::Error<<"Packet for '"<<this->qdomain<<"' denied: "<<err.what()<<endl;
+    g_log<<Logger::Error<<"Packet for '"<<this->qdomain<<"' denied: "<<err.what()<<endl;
     return false;
   }
 
