@@ -23,6 +23,10 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#if HAVE_BOOST_GE_148
+#include "histog.hh"
+#endif
+
 #include "statbag.hh"
 #include "dnspcap.hh"
 #include "dnsparser.hh"
@@ -35,11 +39,10 @@
 #include <algorithm>
 #include "anadns.hh"
 #include <boost/program_options.hpp>
-
+#include <unordered_set>
 #include <boost/logic/tribool.hpp>
 #include "arguments.hh"
 #include "namespaces.hh"
-#include <deque>
 #include "dnsrecords.hh"
 #include "statnode.hh"
 
@@ -120,6 +123,13 @@ void visitor(const StatNode* node, const StatNode::Stat& selfstat, const StatNod
   }
 }
 
+const struct timeval operator-(const struct pdns_timeval& lhs, const struct pdns_timeval& rhs)
+{
+  struct timeval a{lhs.tv_sec, static_cast<suseconds_t>(lhs.tv_usec)}, b{rhs.tv_sec, static_cast<suseconds_t>(rhs.tv_usec)};
+  return operator-(a,b);
+}
+
+
 int main(int argc, char** argv)
 try
 {
@@ -130,8 +140,15 @@ try
     ("rd", po::value<bool>(), "If set to true, only process RD packets, to false only non-RD, unset: both")
     ("ipv4", po::value<bool>()->default_value(true), "Process IPv4 packets")
     ("ipv6", po::value<bool>()->default_value(true), "Process IPv6 packets")
-    ("servfail-tree", "Figure out subtrees that generate servfails")
+#if HAVE_BOOST_GE_148
+    ("log-histogram", "Write a log-histogram to file 'log-histogram'")
+    ("full-histogram", po::value<double>(), "Write a log-histogram to file 'full-histogram' with this millisecond bin size")
+#endif
+    ("filter-name,f", po::value<string>(), "Do statistics only for queries within this domain")
     ("load-stats,l", po::value<string>()->default_value(""), "if set, emit per-second load statistics (questions, answers, outstanding)")
+    ("no-servfail-stats", "Don't include servfails in response time stats")
+    ("servfail-tree", "Figure out subtrees that generate servfails")
+    ("stats-dir", po::value<string>()->default_value("."), "Directory where statistics will be saved")
     ("write-failures,w", po::value<string>()->default_value(""), "if set, write weird packets to this PCAP file")
     ("verbose,v", "be verbose");
     
@@ -156,11 +173,16 @@ try
   }
 
   if(files.empty() || g_vm.count("help")) {
-    cerr<<"Syntax: dnsscope filename.pcap"<<endl;
+    cerr<<"Syntax: dnsscope filename.pcap [filenam2.pcap...]"<<endl;
     cout << desc << endl;
     exit(0);
   }
 
+  DNSName filtername;
+  if(g_vm.count("filter-name"))
+    filtername = DNSName(g_vm["filter-name"].as<string>());
+  uint32_t nameMismatch = 0;
+
   StatNode root;
 
   bool verbose = g_vm.count("verbose");
@@ -177,24 +199,27 @@ try
   bool doIPv4 = g_vm["ipv4"].as<bool>();
   bool doIPv6 = g_vm["ipv6"].as<bool>();
   bool doServFailTree = g_vm.count("servfail-tree");
-  int dnserrors=0, bogus=0;
+  bool noservfailstats = g_vm.count("no-servfail-stats");
+  int dnserrors=0, parsefail=0;
   typedef map<uint32_t,uint32_t> cumul_t;
   cumul_t cumul;
-  unsigned int untracked=0, errorresult=0, reallylate=0, nonRDQueries=0, queries=0;
+  unsigned int untracked=0, errorresult=0, nonRDQueries=0, queries=0;
   unsigned int ipv4DNSPackets=0, ipv6DNSPackets=0, fragmented=0, rdNonRAAnswers=0;
   unsigned int answers=0, nonDNSIP=0, rdFilterMismatch=0;
   unsigned int dnssecOK=0, edns=0;
   unsigned int dnssecCD=0, dnssecAD=0;
+  unsigned int reuses=0;
   typedef map<uint16_t,uint32_t> rcodes_t;
   rcodes_t rcodes;
   
   time_t lowestTime=2000000000, highestTime=0;
   time_t lastsec=0;
   LiveCounts lastcounts;
-  set<ComboAddress, ComboAddress::addressOnlyLessThan> requestors, recipients, rdnonra;
+  std::unordered_set<ComboAddress, ComboAddress::addressOnlyHash> requestors, recipients, rdnonra;
   typedef vector<pair<time_t, LiveCounts> > pcounts_t;
   pcounts_t pcounts;
   OPTRecordContent::report();
+
   for(unsigned int fno=0; fno < files.size(); ++fno) {
     PcapPacketReader pr(files[fno]);
     PcapPacketWriter* pw=0;
@@ -217,22 +242,33 @@ try
 	      continue;
 	    }
 	  }
-	  MOADNSParser mdp(false, (const char*)pr.d_payload, pr.d_len);
-	  if(haveRDFilter && mdp.d_header.rd != rdFilter) {
+	  uint16_t qtype;
+	  DNSName qname((const char*)pr.d_payload, pr.d_len, 12, false, &qtype);
+	  struct dnsheader header;
+	  memcpy(&header, (struct dnsheader*)pr.d_payload, 12);
+
+	  if(haveRDFilter && header.rd != rdFilter) {
 	    rdFilterMismatch++;
 	    continue;
 	  }
-
-	  if(!mdp.d_header.qr && getEDNSOpts(mdp, &edo)) {
-	    edns++;
-	    if(edo.d_Z & EDNSOpts::DNSSECOK)
-	      dnssecOK++;
-	    if(mdp.d_header.cd)
-	      dnssecCD++;
-	    if(mdp.d_header.ad)
-	      dnssecAD++;
-	  }
-	  
+          
+          if(!filtername.empty() && !qname.isPartOf(filtername)) {
+            nameMismatch++;
+            continue;
+          }
+          
+	  if(!header.qr) {
+            uint16_t udpsize, z;
+            if(getEDNSUDPPayloadSizeAndZ((const char*)pr.d_payload, pr.d_len, &udpsize, &z)) {
+              edns++;
+              if(z & EDNSOpts::DNSSECOK)
+                dnssecOK++;
+              if(header.cd)
+                dnssecCD++;
+              if(header.ad)
+                dnssecAD++;
+            }
+          }
 
 	  if(pr.d_ip->ip_v == 4) 
 	    ++ipv4DNSPackets;
@@ -257,12 +293,11 @@ try
 	  lowestTime=min((time_t)lowestTime,  (time_t)pr.d_pheader.ts.tv_sec);
 	  highestTime=max((time_t)highestTime, (time_t)pr.d_pheader.ts.tv_sec);
 
-	  string name=mdp.d_qname.toString()+"|"+DNSRecordContent::NumberToType(mdp.d_qtype);
-        
-	  QuestionIdentifier qi=QuestionIdentifier::create(pr.getSource(), pr.getDest(), mdp);
+	  QuestionIdentifier qi=QuestionIdentifier::create(pr.getSource(), pr.getDest(), header, qname, qtype);
 
-	  if(!mdp.d_header.qr) { // question
-	    if(!mdp.d_header.rd)
+	  if(!header.qr) { // question
+	    //	    cout<<"Query "<<qi<<endl;
+	    if(!header.rd)
 	      nonRDQueries++;
 	    queries++;
 
@@ -270,64 +305,71 @@ try
 	    rem.sin4.sin_port=0;
 	    requestors.insert(rem);	  
 
-	    QuestionData& qd=statmap[qi];
+            QuestionData& qd=statmap[qi];
           
 	    if(!qd.d_firstquestiontime.tv_sec)
 	      qd.d_firstquestiontime=pr.d_pheader.ts;
-	    qd.d_qcount++;
+	    else {
+	      auto delta=makeFloat(pr.d_pheader.ts - qd.d_firstquestiontime);
+	      //	      cout<<"Reuse of "<<qi<<", delta t="<<delta<<", count="<<qd.d_qcount<<endl;
+	      if(delta > 2.0) {
+		//		cout<<"Resetting old entry for "<<qi<<", too old"<<endl;
+		qd.d_qcount=0;
+		qd.d_answercount=0;
+		qd.d_firstquestiontime=pr.d_pheader.ts;
+	      }
+	    }
+	    if(qd.d_qcount++)
+              reuses++;
 	  }
 	  else  {  // answer
-	    rcodes[mdp.d_header.rcode]++;
+	    //	    cout<<"Response "<<qi<<endl;
+	    rcodes[header.rcode]++;
 	    answers++;
-	    if(mdp.d_header.rd && !mdp.d_header.ra) {
+	    if(header.rd && !header.ra) {
 	      rdNonRAAnswers++;
 	      rdnonra.insert(pr.getDest());
 	    }
 	  
-	    if(mdp.d_header.ra) {
+	    if(header.ra) {
 	      ComboAddress rem = pr.getDest();
 	      rem.sin4.sin_port=0;
 	      recipients.insert(rem);	  
 	    }
 
 	    QuestionData& qd=statmap[qi];
-
-	    if(!qd.d_qcount)
+	    if(!qd.d_qcount) {
+	      //	      cout<<"Untracked answer: "<<qi<<endl;
 	      untracked++;
+	    }
 
 	    qd.d_answercount++;
 
 	    if(qd.d_qcount) {
 	      uint32_t usecs= (pr.d_pheader.ts.tv_sec - qd.d_firstquestiontime.tv_sec) * 1000000 +  
 		(pr.d_pheader.ts.tv_usec - qd.d_firstquestiontime.tv_usec) ;
-	      //            cout<<"Took: "<<usecs<<"usec\n";
-	      if(usecs<2049000)
-		cumul[usecs]++;
-	      else
-		reallylate++;
+
+	      //	      cout<<"Usecs for "<<qi<<": "<<usecs<<endl;
+              if(!noservfailstats || header.rcode != 2)
+                cumul[usecs]++;
             
-	      if(mdp.d_header.rcode != 0 && mdp.d_header.rcode!=3) 
+	      if(header.rcode != 0 && header.rcode!=3) 
 		errorresult++;
 	      ComboAddress rem = pr.getDest();
 	      rem.sin4.sin_port=0;
 
 	      if(doServFailTree)
-		root.submit(mdp.d_qname, mdp.d_header.rcode, rem);
+		root.submit(qname, header.rcode, rem);
 	    }
 
-	    if(!qd.d_qcount || qd.d_qcount == qd.d_answercount)
+	    if(!qd.d_qcount || qd.d_qcount == qd.d_answercount) {
+	      //	      cout<<"Clearing state for "<<qi<<endl<<endl;
 	      statmap.erase(qi);
+	    }
+	    else {
+	      //	      cout<<"State for qi remains open, qcount="<<qd.d_qcount<<", answercount="<<qd.d_answercount<<endl;
+            }
 	  }
-
-        
-	}
-	catch(MOADNSException& mde) {
-	  if(verbose)
-	    cout<<"error parsing packet: "<<mde.what()<<endl;
-	  if(pw)
-	    pw->write();
-	  dnserrors++;
-	  continue;
 	}
 	catch(std::exception& e) {
 	  if(verbose)
@@ -335,7 +377,7 @@ try
 
 	  if(pw)
 	    pw->write();
-	  bogus++;
+	  parsefail++;
 	  continue;
 	}
       }
@@ -346,13 +388,24 @@ try
     cout<<"PCAP contained "<<pr.d_correctpackets<<" correct packets, "<<pr.d_runts<<" runts, "<< pr.d_oversized<<" oversize, "<<pr.d_nonetheripudp<<" non-UDP.\n";
 
   }
+
+  /*
+  cout<<"Open when done: "<<endl;
+  for(const auto& a : statmap) {
+    cout<<a.first<<": qcount="<<a.second.d_qcount<<", answercount="<<a.second.d_answercount<<endl;
+  }
+  */
+  
   cout<<"Timespan: "<<(highestTime-lowestTime)/3600.0<<" hours"<<endl;
 
-  cout<<nonDNSIP<<" non-DNS UDP, "<<dnserrors<<" dns decoding errors, "<<bogus<<" bogus packets"<<endl;
+  cout<<nonDNSIP<<" non-DNS UDP, "<<dnserrors<<" dns decoding errors, "<<parsefail<<" packets failed to parse"<<endl;
   cout<<"Ignored fragment packets: "<<fragmented<<endl;
   cout<<"Dropped DNS packets based on recursion-desired filter: "<<rdFilterMismatch<<endl;
+  if(!filtername.empty())
+    cout <<"Dropped DNS packets because not part of '"<<filtername<<"': "<<nameMismatch << endl;
   cout<<"DNS IPv4: "<<ipv4DNSPackets<<" packets, IPv6: "<<ipv6DNSPackets<<" packets"<<endl;
   cout<<"Questions: "<<queries<<", answers: "<<answers<<endl;
+  cout<<"Reuses of same state entry: "<<reuses<<endl;
   unsigned int unanswered=0;
 
 
@@ -378,57 +431,84 @@ try
 
   uint32_t sum=0;
   //  ofstream stats("stats");
-  uint32_t totpackets=reallylate;
+  uint32_t totpairs=0;
   double tottime=0;
   for(cumul_t::const_iterator i=cumul.begin(); i!=cumul.end(); ++i) {
     //    stats<<i->first<<"\t"<<(sum+=i->second)<<"\n";
-    totpackets+=i->second;
+    totpairs+=i->second;
     tottime+=i->first*i->second;
   }
   
   typedef map<uint32_t, bool> done_t;
   done_t done;
-  done[50];
-  done[100];
-  done[200];
-  done[300];
-  done[400];
-  done[800];
-  done[1000];
-  done[2000];
-  done[4000];
-  done[8000];
-  done[32000];
-  done[64000];
-  done[256000];
-  done[1024000];
-  done[2048000];
+  for(auto a : {50, 100, 200, 300, 400, 800, 1000, 2000, 4000, 8000, 32000, 64000, 256000, 1024000, 2048000})
+    done[a]=false;
 
   cout.setf(std::ios::fixed);
-  cout.precision(2);
+  cout.precision(4);
   sum=0;
-  
-  double lastperc=0, perc=0;
-  for(cumul_t::const_iterator i=cumul.begin(); i!=cumul.end(); ++i) {
-    sum+=i->second;
 
-    for(done_t::iterator j=done.begin(); j!=done.end(); ++j)
+#if HAVE_BOOST_GE_148
+  if(g_vm.count("log-histogram")) {
+    string fname = g_vm["stats-dir"].as<string>()+"/log-histogram";
+    ofstream loglog(fname);
+    if(!loglog)
+      throw runtime_error("Unable to write statistics to "+fname);
+
+    writeLogHistogramFile(cumul, loglog);
+  }
+
+  if(g_vm.count("full-histogram")) {
+    string fname=g_vm["stats-dir"].as<string>()+"/full-histogram";
+    ofstream loglog(fname);
+    if(!loglog)
+      throw runtime_error("Unable to write statistics to "+fname);
+    writeFullHistogramFile(cumul, g_vm["full-histogram"].as<double>(), loglog);
+  }
+#endif
+
+  
+  sum=0;
+  double lastperc=0, perc=0;
+  uint64_t lastsum=0;
+
+  for(cumul_t::const_iterator i=cumul.begin(); i!=cumul.end(); ++i) {
+    for(done_t::iterator j=done.begin(); j!=done.end(); ++j) {
       if(!j->second && i->first > j->first) {
         j->second=true;
 
-        perc=sum*100.0/totpackets;
+        perc=sum*100.0/totpairs;
         if(j->first < 1024)
           cout<< perc <<"% of questions answered within " << j->first << " usec (";
         else
           cout<< perc <<"% of questions answered within " << j->first/1000.0 << " msec (";
         
         cout<<perc-lastperc<<"%)\n";
-        lastperc=sum*100.0/totpackets;
+        lastperc=sum*100.0/totpairs;
+        lastsum=sum;
       }
+    }
+    sum+=i->second;
   }
-  cout<<reallylate<<" responses ("<<reallylate*100.0/answers<<"%) older than 2 seconds"<<endl;
-  if(totpackets)
-    cout<<"Average non-late response time: "<<tottime/totpackets<<" usec"<<endl;
+
+  for(auto j = done.begin(); j != done.end(); ++j) {
+    if(!j->second) {
+      perc=sum*100.0/totpairs;
+      if(j->first < 1024)
+        cout<< perc <<"% of questions answered within " << j->first << " usec (";
+      else
+        cout<< perc <<"% of questions answered within " << j->first/1000.0 << " msec (";
+      
+      cout<<perc-lastperc<<"%)\n";
+      lastperc=sum*100.0/totpairs;
+      lastsum=sum;
+      break;
+    }
+  }
+  
+  cout<< (totpairs-lastsum)<<" responses ("<<((totpairs-lastsum)*100.0/answers) <<"%) older than "<< (done.rbegin()->first/1000000.0) <<" seconds"<<endl;
+  if(totpairs)
+    cout<<"Average non-late response time: "<<tottime/totpairs<<" usec"<<endl;
 
   if(!g_vm["load-stats"].as<string>().empty()) {
     ofstream load(g_vm["load-stats"].as<string>().c_str());
